EACCES = 13
EADDRINUSE = 98
EAGAIN = 11
EALREADY = 114
EBADF = 9
ECONNABORTED = 103
ECONNREFUSED = 111
ECONNRESET = 104
EEXIST = 17
EHOSTUNREACH = 113
EINPROGRESS = 115
EINVAL = 22
EIO = 5
EISDIR = 21
ENOBUFS = 105
ENODEV = 19
ENOENT = 2
ENOMEM = 12
ENOTCONN = 107
EOPNOTSUPP = 95
EPERM = 1
ETIMEDOUT = 110
errorcode = None
