
class OneWire:
    ''
    MATCH_ROM = 85
    SEARCH_ROM = 240
    SKIP_ROM = 204
    def _search_rom():
        pass

    def crc8():
        pass

    def readbit():
        pass

    def readbyte():
        pass

    def readinto():
        pass

    def reset():
        pass

    def scan():
        pass

    def select_rom():
        pass

    def write():
        pass

    def writebit():
        pass

    def writebyte():
        pass


class OneWireError:
    ''
_ow = None
def const():
    pass
